@@ -1,16 +1,19 @@
 #include <libavformat/avformat.h>
 #include <libavcodec/avcodec.h>
-#include <libavcodec/avfft.h>
 #include <libavutil/audio_fifo.h>
 #include <libswresample/swresample.h>
 #include "avcompat.h"
 
 #ifdef NEW_FFT_TX_API
 #include <libavutil/tx.h>
+typedef float FFTSample;
+typedef struct FFTComplex {
+  FFTSample re, im;
+} FFTComplex;
+#else
+#include <libavcodec/avfft.h>
 #endif
 
-#define R_NO_REMAP
-#define STRICT_R_HEADERS
 #include <Rinternals.h>
 
 enum AmplitudeScale { AS_LINEAR, AS_SQRT, AS_CBRT, AS_LOG, NB_ASCALES };
@@ -84,7 +87,6 @@ static void close_input(input_container **x){
   input_container *input = *x;
   if(input == NULL)
     return;
-  avcodec_close(input->decoder);
   avcodec_free_context(&(input->decoder));
   avformat_close_input(&input->demuxer);
   avformat_free_context(input->demuxer);
