@@ -5,13 +5,14 @@
 #include <libavutil/channel_layout.h>
 #include <libavutil/opt.h>
 
-#define R_NO_REMAP
-#define STRICT_R_HEADERS
 #define PTS_EVERYTHING 1e18
 #define VIDEO_TIME_BASE 1000
 #include <Rinternals.h>
 #include "avcompat.h"
 
+enum AVPixelFormat get_default_pix_fmt(const AVCodec *codec);
+enum AVSampleFormat get_default_sample_fmt(const AVCodec *codec);
+
 int total_open_handles = 0;
 
 typedef struct {
@@ -48,6 +49,8 @@ typedef struct {
   int progress_pct;
   int channels;
   int sample_rate;
+  int bit_rate;
+  int early_end;
   SEXP in_files;
 } output_container;
 
@@ -78,7 +81,6 @@ static void close_input(input_container **x){
   input_container *input = *x;
   if(input == NULL)
     return;
-  avcodec_close(input->decoder);
   avcodec_free_context(&(input->decoder));
   avformat_close_input(&input->demuxer);
   avformat_free_context(input->demuxer);
@@ -112,12 +114,10 @@ static void close_output_file(void *ptr, Rboolean jump){
   }
   if(output->video_encoder != NULL){
     close_filter_container(output->video_filter);
-    avcodec_close(output->video_encoder);
     avcodec_free_context(&(output->video_encoder));
   }
   if(output->audio_encoder != NULL){
     close_filter_container(output->audio_filter);
-    avcodec_close(output->audio_encoder);
     avcodec_free_context(&(output->audio_encoder));
   }
   if(output->muxer != NULL){
@@ -161,9 +161,17 @@ static int find_stream_audio(AVFormatContext *demuxer, const char *file){
   return out;
 }
 
-static input_container *open_audio_input(const char *filename){
+static input_container *open_audio_input(SEXP audio){
+  const char *filename = CHAR(STRING_ELT(audio, 0));
+  const char *fmt = NULL;
+  int channels = 0;
+  if(Rf_inherits(audio, "pcm")){
+    fmt = CHAR(Rf_asChar(Rf_getAttrib(audio, Rf_install("fmt"))));
+    channels = Rf_asInteger(Rf_getAttrib(audio, Rf_install("channels")));
+  }
   AVFormatContext *demuxer = NULL;
-  bail_if(avformat_open_input(&demuxer, filename, NULL, NULL), "avformat_open_input");
+  const AVInputFormat *pcm_format = fmt ? av_find_input_format(fmt) : NULL;
+  bail_if(avformat_open_input(&demuxer, filename, pcm_format, NULL), "avformat_open_input");
   bail_if(avformat_find_stream_info(demuxer, NULL), "avformat_find_stream_info");
 
   /* Try all input streams */
@@ -175,9 +183,13 @@ static input_container *open_audio_input(const char *filename){
   bail_if(avcodec_parameters_to_context(decoder, stream->codecpar), "avcodec_parameters_to_context");
   bail_if(avcodec_open2(decoder, codec, NULL), "avcodec_open2 (audio)");
 #ifdef NEW_CHANNEL_API
+  if(channels)
+    av_channel_layout_default(&decoder->ch_layout, channels); //PCM input
   if (decoder->ch_layout.order == AV_CHANNEL_ORDER_UNSPEC)
-    av_channel_layout_default(&decoder->ch_layout, decoder->ch_layout.nb_channels);
+    av_channel_layout_default(&decoder->ch_layout, decoder->ch_layout.nb_channels); //other stuff
 #else
+  if(channels)
+    decoder->channels = channels;
   if(!decoder->channel_layout)
     decoder->channel_layout = av_get_default_channel_layout(decoder->channels);
 #endif
@@ -210,9 +222,16 @@ static filter_container *open_audio_filter(AVCodecContext *decoder, AVCodecConte
                                        input_args, NULL, filter_graph), "avfilter_graph_create_filter (audio/src)");
 
   /* Initiate sink filter */
-  AVFilterContext *buffersink_ctx = NULL;
-  bail_if(avfilter_graph_create_filter(&buffersink_ctx, avfilter_get_by_name("abuffersink"), "audiosink",
-                                       NULL, NULL, filter_graph), "avfilter_graph_create_filter (audio/sink)");
+  AVFilterContext *buffersink_ctx = avfilter_graph_alloc_filter(filter_graph,avfilter_get_by_name("abuffersink"), "audiosink");
+
+#ifdef NEW_ARRAY_PARAMS
+  bail_if(av_opt_set_array(buffersink_ctx, "sample_formats", AV_OPT_SEARCH_CHILDREN,
+                           0, 1, AV_OPT_TYPE_SAMPLE_FMT, &encoder->sample_fmt),"av_opt_set_array (sample_formats)");
+  bail_if(av_opt_set_array(buffersink_ctx, "channel_layouts", AV_OPT_SEARCH_CHILDREN,
+                           0, 1, AV_OPT_TYPE_CHLAYOUT, &encoder->ch_layout),"av_opt_set_array (channel_layouts)");
+  bail_if(av_opt_set_array(buffersink_ctx, "samplerates", AV_OPT_SEARCH_CHILDREN,
+                           0, 1, AV_OPT_TYPE_INT, &encoder->sample_rate),"av_opt_set_array (samplerates)");
+#else
 
   /* Set output properties (copied from ffmpeg examples/transcoding.c) */
   bail_if(av_opt_set_bin(buffersink_ctx, "sample_fmts",
@@ -230,6 +249,8 @@ static filter_container *open_audio_filter(AVCodecContext *decoder, AVCodecConte
   bail_if(av_opt_set_bin(buffersink_ctx, "sample_rates",
                          (uint8_t*)&encoder->sample_rate, sizeof(encoder->sample_rate),
                          AV_OPT_SEARCH_CHILDREN), "av_opt_set_bin (sample_rates)");
+#endif //NEW_ARRAY_PARAMS
+  bail_if(avfilter_init_str(buffersink_ctx, NULL), "avfilter_graph_create_filter (audio/sink)");
 
   /* Endpoints for the filter graph. */
   AVFilterInOut *outputs = avfilter_inout_alloc();
@@ -269,15 +290,19 @@ static filter_container *open_video_filter(AVFrame * input, enum AVPixelFormat f
                                        input_args, NULL, filter_graph), "avfilter_graph_create_filter (video/src)");
 
   /* Initiate sink filter */
-  AVFilterContext *buffersink_ctx = NULL;
-  bail_if(avfilter_graph_create_filter(&buffersink_ctx, avfilter_get_by_name("buffersink"), "videosink",
-                                       NULL, NULL, filter_graph), "avfilter_graph_create_filter (video/sink)");
+
+  AVFilterContext *buffersink_ctx = avfilter_graph_alloc_filter(filter_graph,avfilter_get_by_name("buffersink"), "videosink");
 
   /* I think this convert output YUV420P (copied from ffmpeg examples/transcoding.c) */
+#ifdef NEW_ARRAY_PARAMS
+  bail_if(av_opt_set_array(buffersink_ctx, "pixel_formats", AV_OPT_SEARCH_CHILDREN,
+                           0, 1, AV_OPT_TYPE_PIXEL_FMT, &fmt), "av_opt_set_array (pixel_formats)");
+#else
   bail_if(av_opt_set_bin(buffersink_ctx, "pix_fmts",
                          (uint8_t*)&fmt, sizeof(fmt),
                          AV_OPT_SEARCH_CHILDREN), "av_opt_set_bin");
-
+#endif
+  bail_if(avfilter_init_str(buffersink_ctx, NULL), "avfilter_graph_create_filter (audio/sink)");
 
   /* Endpoints for the filter graph. */
   AVFilterInOut *outputs = avfilter_inout_alloc();
@@ -313,7 +338,7 @@ static void add_video_output(output_container *output, int width, int height){
   //video_encoder->gop_size = 25; //one keyframe every 25 frames
 
   /* Try to use codec preferred pixel format, otherwise default to YUV420 */
-  video_encoder->pix_fmt = output->codec->pix_fmts ? output->codec->pix_fmts[0] : AV_PIX_FMT_YUV420P;
+  video_encoder->pix_fmt = get_default_pix_fmt(output->codec);
   if (output->muxer->oformat->flags & AVFMT_GLOBALHEADER)
     video_encoder->flags |= AV_CODEC_FLAG_GLOBAL_HEADER;
 
@@ -347,7 +372,8 @@ static void add_audio_output(output_container *container){
   audio_encoder->channel_layout = av_get_default_channel_layout(audio_encoder->channels);
 #endif
   audio_encoder->sample_rate = container->sample_rate ? container->sample_rate : audio_decoder->sample_rate;
-  audio_encoder->sample_fmt = output_codec->sample_fmts[0];
+  audio_encoder->bit_rate = container->bit_rate ? container->bit_rate : audio_decoder->bit_rate;
+  audio_encoder->sample_fmt = get_default_sample_fmt(output_codec);
   audio_encoder->strict_std_compliance = FF_COMPLIANCE_EXPERIMENTAL;
 
   /* Add the audio_stream to the muxer */
@@ -501,6 +527,7 @@ static int encode_output_frames(output_container *output){
     if(ret == AVERROR_EOF){
       bail_if_null(output, "filter did not return any frames");
       bail_if(avcodec_send_frame(output->video_encoder, NULL), "avcodec_send_frame (flush video)");
+      output->early_end = 1; //trim filter can EOF before input is fully drained
     } else {
       bail_if(ret, "av_buffersink_get_frame");
       if(output->muxer == NULL)
@@ -520,7 +547,9 @@ static int encode_output_frames(output_container *output){
  * a copy of that frame when we finalize the video.
  */
 static int feed_to_filter(AVFrame * image, output_container *output){
-  enum AVPixelFormat pix_fmt = output->codec->pix_fmts ? output->codec->pix_fmts[0] : AV_PIX_FMT_YUV420P;
+  if(output->early_end)
+    return 1;
+  enum AVPixelFormat pix_fmt = get_default_pix_fmt(output->codec);
   static AVFrame *previous = NULL;
   if(previous == NULL)
     previous = av_frame_alloc();
@@ -594,7 +623,7 @@ static void read_from_input(const char *filename, output_container *output){
     if(decoder->codec->id == AV_CODEC_ID_PNG || decoder->codec->id == AV_CODEC_ID_MJPEG)
       picture->pict_type = AV_PICTURE_TYPE_NONE;
     feed_to_filter(picture, output);
-  } while(ret != AVERROR_EOF);
+  } while(ret != AVERROR_EOF && !output->early_end);
   close_input(&output->video_input);
 }
 
@@ -630,7 +659,7 @@ SEXP R_encode_video(SEXP in_files, SEXP out_file, SEXP framerate, SEXP vfilter,
 
   /* Start the output video */
   output_container *output = av_mallocz(sizeof(output_container));
-  output->audio_input = Rf_length(audio) ? open_audio_input(CHAR(STRING_ELT(audio, 0))) : NULL;
+  output->audio_input = Rf_length(audio) ? open_audio_input(audio) : NULL;
   output->output_file = CHAR(STRING_ELT(out_file, 0));
   output->duration = VIDEO_TIME_BASE / Rf_asReal(framerate);
   output->filter_string = CHAR(STRING_ELT(vfilter, 0));
@@ -650,15 +679,17 @@ static SEXP encode_audio_input(void *ptr){
 }
 
 SEXP R_convert_audio(SEXP audio, SEXP out_file, SEXP out_format, SEXP out_channels,
-                     SEXP sample_rate, SEXP start_pos, SEXP max_len){
+                     SEXP sample_rate, SEXP bit_rate, SEXP start_pos, SEXP max_len){
   output_container *output = av_mallocz(sizeof(output_container));
   if(Rf_length(out_channels))
     output->channels = Rf_asInteger(out_channels);
   if(Rf_length(sample_rate))
     output->sample_rate = Rf_asInteger(sample_rate);
+  if(Rf_length(bit_rate))
+    output->bit_rate = Rf_asInteger(bit_rate);
   if(Rf_length(out_format))
     output->format_name = CHAR(STRING_ELT(out_format, 0));
-  output->audio_input = open_audio_input(CHAR(STRING_ELT(audio, 0)));
+  output->audio_input = open_audio_input(audio);
   double start_pts = Rf_length(start_pos) ? Rf_asReal(start_pos) : 0;
   if(start_pts > 0)
     av_seek_frame(output->audio_input->demuxer, -1, start_pts * AV_TIME_BASE, AVSEEK_FLAG_ANY);
