@@ -20,3 +20,15 @@ test_that("Listing formats", {
   expect_equal(names(decoders), c("type", "name", "description", "format"))
   expect_equal(names(filters), c("name", "description"))
 })
+
+test_that("Critical encoders", {
+  encoders <- av_encoders()
+  expect_contains(encoders$name, 'libmp3lame')
+  expect_contains(encoders$name, 'libvorbis')
+  expect_contains(encoders$name, 'libxvid')
+  if(!grepl("(Fedora|Rocky|Red Hat)", osVersion)){
+    # libx264 is not supported on RHEL/Fedora libavfilter-free-devel but it is
+    # in ffmpeg-devel from rpmfusion.
+    expect_contains(encoders$name, 'libx264')
+  }
+})
